#========================================================================================
# package      DE_EPAGES::Customer::t::unit::CustomerCollectionAndCreationResource
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Customer::API::Resource::CustomerCollectionAndCreationResource.
#========================================================================================
package DE_EPAGES::Customer::t::unit::CustomerCollectionAndCreationResource;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Customer::API::CustomerService;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_post();
    $self->test_get();

    done_testing();
}



sub test_post {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();

    my $Resource = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');

    my $ahRequestBody = {
        'customerNumber' => '123',
    };

    my $hUpdatedCustomerData = $Resource->post($Shop, $ahRequestBody);

    my $aArguments = GetParametersFromMockifyCall($Mockery->get('CustomerService'), 'createCustomer');
    is_deeply($aArguments->[1], $ahRequestBody, "$SubTestName - test if createCustomer was called with the correct request body");

    is_deeply($hUpdatedCustomerData, $self->_createCustomerData(), "$SubTestName - test if expected customer data was returned.");
}

sub test_get {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createMocks();

    my $Resource = $Mockery->get('SUT');
    my $Shop = $Mockery->get('Shop');
    my $ResultsPerPage = 10;
    my $Page = 1;

    $Resource->get($Shop, $ResultsPerPage, $Page);

    my $Was_GetCustomers_Called = WasCalled($Mockery->get('CustomerService'), 'getCustomers');
    ok($Was_GetCustomers_Called, "$SubTestName - tes if getCustomer was called for CustomerService");
}


sub _createMocks {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['CustomerService', 'Pager'], sub {
        my ($CustomerService, $Pager) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::Resource::CustomerCollectionAndCreationResource', []);
        $Mockify->addMockWithReturnValue('_createCustomerService', $CustomerService);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createResourcePager', $Pager, [{'hashref' => {'Page' => 1, 'PageSize' => 10}}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('CustomerService', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Customer::API::CustomerService', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('createCustomer', $self->_createCustomerData(), ['object', 'hashref']);
        $Mockify->addMockWithReturnValueAndParameterCheck('getCustomerData', undef, ['object']);
        $Mockify->addMockWithReturnValueAndParameterCheck('getCustomers', $self->_getCustomers(), [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});
        return $Mockify->getMockObject();
    });

    $Mockery->register('Pager', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::WebService::API::ResourcePager', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('pagedItems', undef, ['arrayref', 'any']);
        return $Mockify->getMockObject();
    });

    return $Mockery->assemble();
}

sub _getCustomers {
    return ['Customer'];
}


sub _createCustomerData {
    return {
        'id' => 'test customer',
    };
}

__PACKAGE__->runScript();