package DE_EPAGES::MeinPaket::t::unit::MeinPaketOffer;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Test::Mock::Mockery;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_NoProduct();
    $self->test_NoMasterProduct();
    $self->test_MasterProduct_VariantGroupCode();
    $self->test_MasterProduct_UploadAsSeperateProducts();
    $self->test_areStockLevelsValid();
    $self->test_areStockLevelsNotValid();

    done_testing();
}

sub test_NoProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $MeinPaketOffer = $self->_createOfferWithoutProduct();
    my $aProducts = $MeinPaketOffer->getProducts();
    is_deeply($aProducts, [], "$SubTestName - test if we get an empty array when product is not defined");
}

sub test_NoMasterProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hSettings = {
        'IsMasterProduct' => 0
    };
    my $Mockery = $self->_createMocks($hSettings);
    my $aProducts = $Mockery->get('SUT')->getProducts();

    isa_ok(
        $aProducts->[0],
        'DE_EPAGES::Product::API::Object::Product',
        "$SubTestName - test if we get an array with a single element containing a product"
    );

    is(scalar @{$aProducts}, 1, "$SubTestName - array has only one element");
}

sub test_MasterProduct_VariantGroupCode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hSettings = {
        'IsMasterProduct' => 1,
        'VariantGroupCode' => 123456,
        'UploadAsSeperateProducts' => 0,
    };
    my $Mockery = $self->_createMocks($hSettings);
    my $aProducts = $Mockery->get('SUT')->getProducts();

    isa_ok(
        $aProducts->[0],
        'DE_EPAGES::Product::API::Object::Product',
        "$SubTestName - test if we get a product in the first element"
    );
    isa_ok(
        $aProducts->[1],
        'DE_EPAGES::Product::API::Object::Product',
        "$SubTestName - test if we get a product in the second element"
    );

    is(scalar @{$aProducts}, 2, "$SubTestName - array has two elements");
}

sub test_MasterProduct_UploadAsSeperateProducts {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hSettings = {
        'IsMasterProduct' => 1,
        'VariantGroupCode' => undef,
        'UploadAsSeperateProducts' => 1,
    };
    my $Mockery = $self->_createMocks($hSettings);
    my $aProducts = $Mockery->get('SUT')->getProducts();

    isa_ok(
        $aProducts->[0],
        'DE_EPAGES::Product::API::Object::Product',
        "$SubTestName - test if we get a product in the first element"
    );
    isa_ok(
        $aProducts->[1],
        'DE_EPAGES::Product::API::Object::Product',
        "$SubTestName - test if we get a product in the second element"
    );

    is(scalar @{$aProducts}, 2, "$SubTestName - array has two elements");
}

sub test_areStockLevelsValid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hSettings = {
        'IsMasterProduct' => 1,
        'VariantGroupCode' => undef,
        'UploadAsSeperateProducts' => undef,
        'StockLevelSubProduct1' => 2,
        'StockLevelSubProduct2' => 5,
    };
    my $Mockery = $self->_createMocks($hSettings);
    my $SUT = $Mockery->get('SUT');
    my $AreStockLevelsValid = $SUT->areStockLevelsValid();
    is($AreStockLevelsValid, 1, "$SubTestName - check if StockLevels are valid");
}

sub test_areStockLevelsNotValid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hSettings = {
        'IsMasterProduct' => 1,
        'VariantGroupCode' => undef,
        'UploadAsSeperateProducts' => undef,
        'StockLevelSubProduct1' => 3.5,
        'StockLevelSubProduct2' => 5,
    };
    my $Mockery = $self->_createMocks($hSettings);
    my $SUT = $Mockery->get('SUT');
    my $AreStockLevelsValid = $SUT->areStockLevelsValid();
    is($AreStockLevelsValid, 0, "$SubTestName - check if StockLevels are not valid");
}

#---------------------------------------------------------------------------
#  helpers
#---------------------------------------------------------------------------

sub _createMocks {
    my $self = shift;
    my ($hSettings) = @_;
    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Product'], sub {
        my ($Product) = @_;
        my $Object = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::MeinPaket::API::Object::MeinPaketOffer', {
            'Product' => $Product,
            'VariantGroupCode' => $hSettings->{'VariantGroupCode'},
            'UploadAsSeperateProducts' => $hSettings->{'UploadAsSeperateProducts'},
        });
        return $Object->getMockObject();
    });
    $Mockery->register('Product', ['SubProduct1','SubProduct2'], sub {
        my ($SubProduct1,$SubProduct2) = @_;
        my $Object = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'IsMaster' => $hSettings->{'IsMasterProduct'},
            'SubProducts' => [$SubProduct1,$SubProduct2],
        });
        return $Object->getMockObject();
    });
    $Mockery->register('SubProduct1', [], sub {
        my $Object = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'IsMaster' => 0,
            'StockLevel' => $hSettings->{'StockLevelSubProduct1'},
        });
        return $Object->getMockObject();
    });

    $Mockery->register('SubProduct2', [], sub {
        my $Object = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'IsMaster' => 0,
            'StockLevel' => $hSettings->{'StockLevelSubProduct2'},
        });
        return $Object->getMockObject();
    });

    return $Mockery->assemble();
}

sub _createOfferWithoutProduct {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::MeinPaket::API::Object::MeinPaketOffer', {
            'Product' => undef,
            'VariantGroupCode' => undef,
            'UploadAsSeperateProducts' => undef,
        });
        return $Mockify->getMockObject();
}
__PACKAGE__->runScript();