#========================================================================================
# package      DE_EPAGES::MeinPaket::t::unit::OfferUploadResponseHandler
# base         DE_EPAGES::Test::API::UnitTest::EpagesUnitTest
#----------------------------------------------------------------------------------------
# description  Unit test for OfferUploadResponseHandler
#========================================================================================
package DE_EPAGES::MeinPaket::t::unit::OfferUploadResponseHandler;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::MeinPaket::API::OfferUploadResponseHandler;
use DE_EPAGES::MeinPaket::t::unit::MeinPaketMockFactory;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::MeinPaket::API::DTO::Confirmation;
use DE_EPAGES::MeinPaket::API::DTO::UploadResponse;
use DE_EPAGES::MeinPaket::API::DTO::ProductOffer;
use Test::More;
use strict;

sub test {
    my $self = shift;

    $self->prepareMocks();

    $self->test_normalProduct_confirmed();
    $self->test_normalProduct_notConfirmed();
    $self->test_variantionProduct_allConfirmed();
    $self->test_variantionProduct_notAllConfirmed();
    $self->test_variantionProduct_allUploadedConfirmed();
    $self->test_getMeinPaketId();

    done_testing();
}

sub test_normalProduct_confirmed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MeinPaketOffer = $self->memory->get('MeinPaketOffer_OneProduct');
    my $UploadResponse = $self->memory->get('UploadResponse_OneConfirmation');

    my $SUT = DE_EPAGES::MeinPaket::API::OfferUploadResponseHandler->new($MeinPaketOffer, $UploadResponse);

    is($SUT->areAllProductsConfirmed(), 1, "$SubTestName - test if all products are confirmed.");
    is($SUT->getConfirmedProductCount(), 1, "$SubTestName - test if confirmed product count is correct.");
    is($SUT->getUnconfirmedProductCount(), 0, "$SubTestName - test if unconfirmed product count is correct.");
}

sub test_normalProduct_notConfirmed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MeinPaketOffer = $self->memory->get('MeinPaketOffer_OneProduct');
    my $UploadResponse = $self->memory->get('UploadResponse_NoConfirmation');

    my $SUT = DE_EPAGES::MeinPaket::API::OfferUploadResponseHandler->new($MeinPaketOffer, $UploadResponse);

    is($SUT->areAllProductsConfirmed(), 0, "$SubTestName - test if all products are confirmed.");
    is($SUT->getConfirmedProductCount(), 0, "$SubTestName - test if confirmed product count is correct.");
    is($SUT->getUnconfirmedProductCount(), 1, "$SubTestName - test if unconfirmed product count is correct.");
}

sub test_variantionProduct_allConfirmed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MeinPaketOffer = $self->memory->get('MeinPaketOffer_TwoProducts');
    my $UploadResponse = $self->memory->get('UploadResponse_TwoConfirmations');

    my $SUT = DE_EPAGES::MeinPaket::API::OfferUploadResponseHandler->new($MeinPaketOffer, $UploadResponse);

    is($SUT->areAllProductsConfirmed(), 1, "$SubTestName - test if all products are confirmed.");
    is($SUT->getConfirmedProductCount(), 2, "$SubTestName - test if confirmed product count is correct.");
    is($SUT->getUnconfirmedProductCount(), 0, "$SubTestName - test if unconfirmed product count is correct.");
}

sub test_variantionProduct_notAllConfirmed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MeinPaketOffer = $self->memory->get('MeinPaketOffer_TwoProducts');
    my $UploadResponse = $self->memory->get('UploadResponse_OneConfirmation');

    my $SUT = DE_EPAGES::MeinPaket::API::OfferUploadResponseHandler->new($MeinPaketOffer, $UploadResponse);

    is($SUT->areAllProductsConfirmed(), 0, "$SubTestName - test if all products are confirmed.");
    is($SUT->getConfirmedProductCount(), 1, "$SubTestName - test if confirmed product count is correct.");
    is($SUT->getUnconfirmedProductCount(), 1, "$SubTestName - test if unconfirmed product count is correct.");
}

sub test_variantionProduct_allUploadedConfirmed {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MeinPaketOffer = $self->memory->get('MeinPaketOffer_TwoUploaded_OneNotUploaded');
    my $UploadResponse = $self->memory->get('UploadResponse_TwoConfirmations');

    my $SUT = DE_EPAGES::MeinPaket::API::OfferUploadResponseHandler->new($MeinPaketOffer, $UploadResponse);

    is($SUT->areAllProductsConfirmed(), 1, "$SubTestName - test if all products are confirmed.");
    is($SUT->getConfirmedProductCount(), 2, "$SubTestName - test if confirmed product count is correct.");
    is($SUT->getUnconfirmedProductCount(), 0, "$SubTestName - test if unconfirmed product count is correct.");
}

sub test_getMeinPaketId {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MeinPaketOffer = $self->memory->get('MeinPaketOffer_TwoUploaded_OneNotUploaded');
    my $UploadResponse = $self->memory->get('UploadResponse_TwoConfirmations');
    my $UploadedProduct = $self->memory->get('UploadedProductA');
    my $NotUploadedProduct = $self->memory->get('NotUploadedProductC');

    my $SUT = DE_EPAGES::MeinPaket::API::OfferUploadResponseHandler->new($MeinPaketOffer, $UploadResponse);

    is($SUT->getMeinPaketId($UploadedProduct), 'MeinPaket-001', "$SubTestName - test if the mein paket id is returned as expected.");
    is($SUT->getMeinPaketId($NotUploadedProduct), undef, "$SubTestName - test if the mein paket id is returned as expected.");
}

sub prepareMocks {
    my $self = shift;

    my $MockFactory = DE_EPAGES::MeinPaket::t::unit::MeinPaketMockFactory->new();
    $self->setFactory($MockFactory);

    my $UploadedProductA = $self->mockFactory->buildProduct({
        'Alias' => 'ABC-001',
        'MeinPaketID' => 'MeinPaket-001',
    })->getMockObject();

    my $UploadedProductB = $self->mockFactory->buildProduct({
        'Alias' => 'ABC-002',
        'MeinPaketID' => 'MeinPaket-002',
    })->getMockObject();

    my $NotUploadedProductC = $self->mockFactory->buildProduct({
        'Alias' => 'ABC-003',
        'MeinPaketID' => undef,
    })->getMockObject();

    $self->memory->add('UploadedProductA', $UploadedProductA);
    $self->memory->add('UploadedProductB', $UploadedProductB);
    $self->memory->add('NotUploadedProductC', $NotUploadedProductC);

    my $ProductOfferA = $self->_buildProductOfferDTO({
        'productId' => 'ABC-001',
        'meinPaketId' => 'MeinPaket-001',
    });

    my $ProductOfferB = $self->_buildProductOfferDTO({
        'productId' => 'ABC-002',
        'meinPaketId' => 'MeinPaket-002',
    });

    my $ConfirmationA = $self->_buildConfirmationDTO({
        'productOffer' => $ProductOfferA,
    });

    my $ConfirmationB = $self->_buildConfirmationDTO({
        'productOffer' => $ProductOfferB,
    });

    my $UploadResponseNoConfirmations = $self->_buildUploadResponseDTO([]);
    my $UploadResponseOneConfirmation = $self->_buildUploadResponseDTO([$ConfirmationA]);
    my $UploadResponseTwoConfirmations = $self->_buildUploadResponseDTO([$ConfirmationA, $ConfirmationB]);

    $self->memory->add('UploadResponse_NoConfirmation', $UploadResponseNoConfirmations);
    $self->memory->add('UploadResponse_OneConfirmation', $UploadResponseOneConfirmation);
    $self->memory->add('UploadResponse_TwoConfirmations', $UploadResponseTwoConfirmations);

    my $MeinPaketOfferOneProduct = $self->mockFactory->buildMeinPaketOffer({});
    $MeinPaketOfferOneProduct->addMockWithReturnValue('getProducts', [$UploadedProductA]);

    my $MeinPaketOfferTwoProducts = $self->mockFactory->buildMeinPaketOffer({});
    $MeinPaketOfferTwoProducts->addMockWithReturnValue('getProducts', [$UploadedProductA, $UploadedProductB]);

    my $MeinPaketOfferThreeProducts = $self->mockFactory->buildMeinPaketOffer({});
    $MeinPaketOfferThreeProducts->addMockWithReturnValue('getProducts', [$UploadedProductA, $UploadedProductB, $NotUploadedProductC]);

    $self->memory->add('MeinPaketOffer_OneProduct', $MeinPaketOfferOneProduct->getMockObject());
    $self->memory->add('MeinPaketOffer_TwoProducts', $MeinPaketOfferTwoProducts->getMockObject());
    $self->memory->add('MeinPaketOffer_TwoUploaded_OneNotUploaded', $MeinPaketOfferThreeProducts->getMockObject());
}

sub _buildUploadResponseDTO {
    my $self = shift;
    my ($aConfirmations) = @_;

    my $UploadResponse = DE_EPAGES::MeinPaket::API::DTO::UploadResponse->new();

    foreach my $Confirmation ( @{$aConfirmations }) {
        $UploadResponse->pushConfirmation($Confirmation);
    }

    return $UploadResponse;
}

sub _buildConfirmationDTO {
    my $self = shift;
    my ($hAttributes) = @_;

    my $Confirmation = DE_EPAGES::MeinPaket::API::DTO::Confirmation->new();
    $Confirmation->productOffer($hAttributes->{'productOffer'});

    return $Confirmation;
}

sub _buildProductOfferDTO {
    my $self = shift;
    my ($hAttributes) = @_;

    my $ProductOffer = DE_EPAGES::MeinPaket::API::DTO::ProductOffer->new();
    $ProductOffer->productId($hAttributes->{'productId'});
    $ProductOffer->meinPaketId($hAttributes->{'meinPaketId'});

    return $ProductOffer;
}

__PACKAGE__->runScript();