#========================================================================================
# package      DE_EPAGES::MeinPaket::t::unit::UIErrorCollector
# base         base DE_EPAGES::Core::API::Test
#----------------------------------------------------------------------------------------
# description  Unit test for UIErrorCollector
#========================================================================================
package DE_EPAGES::MeinPaket::t::unit::UIErrorCollector;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::MeinPaket::API::UIErrorCollector;
use DE_EPAGES::MeinPaket::API::DTO::Error;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->test_processError();

    done_testing();
}

sub test_processError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::MeinPaket::API::UIErrorCollector->new();
    my $Warning = $self->_buildWarning({
        'Code' => 'SERVER_ERROR',
        'Message' => 'SomeMessage',
        'Offer' => '123',
        'Product' => '234',
    });
    my $WarningUndefinedError = $self->_buildWarning({
        'Code' => 'SOME_ERROR',
        'Message' => 'SomeMessage',
        'Offer' => '123',
        'Product' => '234',
    });
    my $ErrorNativeDie = $self->_buildError('NativeDie', 'Something bad happend');
    my $ErrorNativeWarn = $self->_buildError('NativeWarn', 'Something not that bad happend');

    lives_ok(sub {
        $SUT->processError($Warning);
    },  "$SubTestName - test if proccess error lives");

    lives_ok(sub {
        $SUT->processError($WarningUndefinedError);
    },  "$SubTestName - test if proccess error lives with an undefined error");

    lives_ok(sub {
        $SUT->processError($ErrorNativeDie);
    },  "$SubTestName - test if proccess error lives with an native die");

    lives_ok(sub {
        $SUT->processError($ErrorNativeWarn);
    },  "$SubTestName - test if proccess error lives with an native warn");

    my $aUIErrors = $SUT->getErrors();

    is($aUIErrors->[0]->{'Code'}, 'Allyouneed Server Fehler.', "$SubTestName - test if code is translated correct to 1st Error");
    is($aUIErrors->[0]->{'Message'}, 'SomeMessage', "$SubTestName- test if message is transfered correct to 1st Error");
    is($aUIErrors->[0]->{'ErrorOffer'}, '123', "$SubTestName- test if offer is transfered correct to 1st Error");
    is($aUIErrors->[0]->{'ErrorProduct'}, '234', "$SubTestName- test if product is transfered correct to 1st Error");

    is($aUIErrors->[1]->{'Code'}, 'SOME_ERROR', "$SubTestName - test if unknown code is transfered correct to 2nd Error");
    is($aUIErrors->[1]->{'Message'}, 'SomeMessage', "$SubTestName- test if message is transfered correct to 2nd Error");
    is($aUIErrors->[1]->{'ErrorOffer'}, '123', "$SubTestName- test if offer is transfered correct to 2nd Error");
    is($aUIErrors->[1]->{'ErrorProduct'}, '234', "$SubTestName- test if product is transfered correct to 2nd Error");

    is($aUIErrors->[2]->{'Code'}, 'NativeDie', "$SubTestName - test if native die error code is transfered correct to 3rd Error");
    is($aUIErrors->[2]->{'Message'}, 'Something bad happend', "$SubTestName- test if message is transfered correct to 3rd Error");

    is($aUIErrors->[3], undef, "$SubTestName- test if NativeWarn was ignored");
}

sub _buildErrorDTO {
    my $self = shift;
    my ($hAttributes) = @_;

    my $ErrorDTO = DE_EPAGES::MeinPaket::API::DTO::Error->new();
    $ErrorDTO->errorMessage($hAttributes->{'Message'});
    $ErrorDTO->errorCode($hAttributes->{'Code'});

    return $ErrorDTO;
}

sub _buildOffer {
    my $self = shift;
    my ($Offer) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::MeinPaket::API::Object::MeinPaketOffer', {});
    $Mockify->addMockWithReturnValue('tleHash', $Offer);

    return $Mockify->getMockObject();
}

sub _buildProduct {
    my $self = shift;
    my ($Product) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {});
    $Mockify->addMockWithReturnValue('tleHash', $Product);

    return $Mockify->getMockObject();
}


sub _buildWarning {
    my $self = shift;
    my ($hAttributes) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Core::API::Warning',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('vars', {
        'Error' => $self->_buildErrorDTO($hAttributes),
        'Offer' => $self->_buildOffer($hAttributes->{'Offer'}),
        'Product' => $self->_buildProduct($hAttributes->{'Product'}),
    });

    return $Mockify->getMockObject();
}

sub _buildError {
    my $self = shift;
    my ($Code, $Message) = @_;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::Core::API::Error',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('code', $Code);
    $Mockify->addMockWithReturnValue('message', $Message);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();