#========================================================================================
# package      DE_EPAGES::Test::t::EpagesObjectMockFactory
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Test::Mock::EpagesObjectMockFactory.
#========================================================================================
package DE_EPAGES::Test::t::EpagesObjectMockFactory;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::EpagesObjectMockFactory;
use Test::Exception;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_build();
    $self->test_buildAddress();
    $self->test_buildBasket();
    $self->test_buildClass();
    $self->test_buildCustomer();
    $self->test_buildCustomerAttribute();
    $self->test_buildCustomerOrder();
    $self->test_buildLineItemContainer();
    $self->test_buildLineItemCoupon();
    $self->test_buildLineItemEcoParticipation();
    $self->test_buildLineItemPayment();
    $self->test_buildLineItemPaymentDiscount();
    $self->test_buildLineItemProduct();
    $self->test_buildLineItemShipping();
    $self->test_buildPaymentType();
    $self->test_buildProduct();
    $self->test_buildShippingFulfillmentType();
    $self->test_buildShippingOption();
    $self->test_buildShippingProviderType();
    $self->test_buildShop();
    $self->test_buildSite();
    $self->test_buildUser();

    done_testing();
}

sub test_build {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();
    my $ShopMock = $SUT->build('DE_EPAGES::Shop::API::Object::Shop', {
        'Alias' => 'ShopMock',
        'SomeParam' => 'SomeValue',
    });

    my $Shop = $ShopMock->getMockObject();

    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if mock has the expected type.");

    is($Shop->alias(), 'ShopMock', "$SubTestName - test if mock has the expected alias.");
    is($Shop->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildAddress({
        'Alias' => 'AddressMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Address::API::Object::Address', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'AddressMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildBasket({
        'Alias' => 'BasketMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Order::API::Object::Basket', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'BasketMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildClass {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildClass({
        'Alias' => 'ClassMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Object::API::Object::Class', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'ClassMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildCustomer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildCustomer({
        'Alias' => 'CustomerMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Customer::API::Object::Customer', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'CustomerMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildCustomerAttribute {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildCustomerAttribute({
        'Alias' => 'CustomerAttributeMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Customer::API::Object::CustomerAttribute', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'CustomerAttributeMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildCustomerOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildCustomerOrder({
        'Alias' => 'CustomerOrderMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Order::API::Object::CustomerOrder', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'CustomerOrderMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildLineItemContainer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildLineItemContainer({
        'Alias' => 'LineItemContainerMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Order::API::Object::LineItemContainer', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'LineItemContainerMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildLineItemCoupon {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildLineItemCoupon({
        'Alias' => 'LineItemCouponMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Coupon::API::Object::LineItemCoupon', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'LineItemCouponMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildLineItemEcoParticipation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildLineItemEcoParticipation({
        'Alias' => 'LineItemEcoMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Product::API::Object::LineItemEcoParticipation', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'LineItemEcoMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildLineItemPayment {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildLineItemPayment({
        'Alias' => 'LineItemPaymentMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Order::API::Object::LineItemPayment', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'LineItemPaymentMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildLineItemPaymentDiscount {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildLineItemPaymentDiscount({
        'Alias' => 'LineItemPaymentDiscountMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Order::API::Object::LineItemPaymentDiscount', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'LineItemPaymentDiscountMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildLineItemProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildLineItemProduct({
        'Alias' => 'LineItemProductMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Product::API::Object::LineItemProduct', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'LineItemProductMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildLineItemShipping {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildLineItemShipping({
        'Alias' => 'LineItemShippingMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Order::API::Object::LineItemShipping', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'LineItemShippingMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildPaymentType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildPaymentType({
        'Alias' => 'PaymentTypeMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Payment::API::Object::PaymentType', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'PaymentTypeMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildProduct({
        'Alias' => 'ProductMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Product::API::Object::Product', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'ProductMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildShippingFulfillmentType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildShippingFulfillmentType({
        'Alias' => 'ShippingFulfillmentTypeMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'ShippingFulfillmentTypeMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildShippingOption {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildShippingOption({
        'Alias' => 'ShippingOptionMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::ShippingOption::API::Object::ShippingOption', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'ShippingOptionMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildShippingProviderType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildShippingProviderType({
        'Alias' => 'ShippingProviderTypeMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::ExternalShipping::API::Object::ShippingProviderType', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'ShippingProviderTypeMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildShop({
        'Alias' => 'ShopMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'ShopMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildSite {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildSite({
        'Alias' => 'SiteMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Object::API::Object::Site', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'SiteMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

sub test_buildUser {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new();

    my $Mock = $SUT->buildUser({
        'Alias' => 'UserMock',
        'SomeParam' => 'SomeValue',
    });

    my $Object = $Mock->getMockObject();

    isa_ok($Object, 'DE_EPAGES::Permission::API::Object::User', "$SubTestName - test if mock has the expected type.");

    is($Object->alias(), 'UserMock', "$SubTestName - test if mock has the expected alias.");
    is($Object->get('SomeParam'), 'SomeValue', "$SubTestName - test if mock has set params.");
}

__PACKAGE__->runScript();