#========================================================================================
# package      DE_EPAGES::Test::t::unit::MockMemory
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Test::Mock::MockMemory.
#========================================================================================
package DE_EPAGES::Test::t::unit::MockMemory;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockMemory;
use Test::Exception;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_addAndGet();
    $self->test_add_duplicateKey();
    $self->test_get_nonExistingKey();
    $self->test_replace();
    $self->test_replace_nonExistingKey();
    $self->test_clear();

    done_testing();
}

sub test_addAndGet {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::MockMemory->new();

    $SUT->add('Key1', 'Value1');
    $SUT->add('Key2', 'Value2');

    is($SUT->get('Key1'), 'Value1', "$SubTestName - test save and get method");
    is($SUT->get('Key2'), 'Value2', "$SubTestName - test save and get method");
}

sub test_add_duplicateKey {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::MockMemory->new();

    $SUT->add('Key1', 'Value1');

    throws_ok(sub { $SUT->add('Key1', 'Value2') }, qr/already exists/, "$SubTestName - test if error is thrown after attempt to save a duplicate key.");
}

sub test_get_nonExistingKey {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::MockMemory->new();

    throws_ok(sub { $SUT->get('Key1'); }, qr/does not exist/, "$SubTestName - test if error is thrown after attempt to get a non existing key.");
}

sub test_replace {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::MockMemory->new();

    $SUT->add('Key1', 'Value1');
    $SUT->replace('Key1', 'Value2');

    is($SUT->get('Key1'), 'Value2', "$SubTestName - test replace method");
}

sub test_replace_nonExistingKey {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::MockMemory->new();

    throws_ok(sub {  $SUT->replace('Key1', 'Value2'); }, qr/does not exist/, "$SubTestName - test if error is thrown after attempt to replace a non existing key.");
}

sub test_clear {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = DE_EPAGES::Test::Mock::MockMemory->new();

    $SUT->add('Key1', 'Value1');
    is($SUT->get('Key1'), 'Value1', "$SubTestName - test if value was stored");

    $SUT->clear();
    throws_ok(sub { $SUT->get('Key1'); }, qr/does not exist/, "$SubTestName - test if error is thrown after attempt to get a non existing key.");
}

__PACKAGE__->runScript();